import { assert } from "../utils/assert.js";
import { decompressReadStream, concatenateReadStream } from "../filesystems/stream-utils.node.js";
import { Headers } from "./headers-polyfill.js";
const isBoolean = x => typeof x === 'boolean';
const isFunction = x => typeof x === 'function';
const isObject = x => x !== null && typeof x === 'object';
const isReadableNodeStream = x => isObject(x) && isFunction(x.read) && isFunction(x.pipe) && isBoolean(x.readable);
import * as stream from 'stream';
export class Response {
  constructor(body, options) {
    this.ok = void 0;
    this.status = void 0;
    this.statusText = void 0;
    this.headers = void 0;
    this.url = void 0;
    this.bodyUsed = false;
    this._body = void 0;
    const {
      headers,
      status = 200,
      statusText = 'OK',
      url
    } = options || {};
    this.url = url;
    this.ok = status === 200;
    this.status = status;
    this.statusText = statusText;
    this.headers = new Headers((options === null || options === void 0 ? void 0 : options.headers) || {});
    if (isReadableNodeStream(body)) {
      this._body = decompressReadStream(body, headers);
    } else if (typeof body === 'string') {
      this._body = stream.Readable.from([new TextEncoder().encode(body)]);
    } else {
      this._body = stream.Readable.from([body || new ArrayBuffer(0)]);
    }
  }
  get body() {
    assert(!this.bodyUsed);
    assert(isReadableNodeStream(this._body));
    this.bodyUsed = true;
    return this._body;
  }
  async arrayBuffer() {
    if (!isReadableNodeStream(this._body)) {
      return this._body || new ArrayBuffer(0);
    }
    const data = await concatenateReadStream(this._body);
    return data;
  }
  async text() {
    const arrayBuffer = await this.arrayBuffer();
    const textDecoder = new TextDecoder();
    return textDecoder.decode(arrayBuffer);
  }
  async json() {
    const text = await this.text();
    return JSON.parse(text);
  }
  async blob() {
    if (typeof Blob === 'undefined') {
      throw new Error('Blob polyfill not installed');
    }
    return new Blob([await this.arrayBuffer()]);
  }
}
//# sourceMappingURL=response-polyfill.js.map