import { resolvePath } from '@loaders.gl/loader-utils';
import fs from 'fs';
export class NodeFile {
  constructor(path, flags, mode) {
    this.handle = void 0;
    this.size = void 0;
    this.bigsize = void 0;
    this.url = void 0;
    path = resolvePath(path);
    this.handle = fs.openSync(path, flags, mode);
    const stats = fs.fstatSync(this.handle, {
      bigint: true
    });
    this.size = Number(stats.size);
    this.bigsize = stats.size;
    this.url = path;
  }
  async close() {
    return new Promise((resolve, reject) => {
      fs.close(this.handle, err => err ? reject(err) : resolve());
    });
  }
  async stat() {
    return await new Promise((resolve, reject) => fs.fstat(this.handle, {
      bigint: true
    }, (err, info) => {
      const stats = {
        size: Number(info.size),
        bigsize: info.size,
        isDirectory: info.isDirectory()
      };
      if (err) {
        reject(err);
      } else {
        resolve(stats);
      }
    }));
  }
  async read(offset, length) {
    const arrayBuffer = new ArrayBuffer(length);
    let bigOffset = BigInt(offset);
    let totalBytesRead = 0;
    const uint8Array = new Uint8Array(arrayBuffer);
    let position;
    while (length > 0) {
      const bytesRead = await readBytes(this.handle, uint8Array, 0, length, bigOffset);
      if (bytesRead === 0) {
        break;
      }
      totalBytesRead += bytesRead;
      bigOffset += BigInt(bytesRead);
      length -= bytesRead;
      if (position !== undefined) {
        position += bytesRead;
      }
    }
    return totalBytesRead < length ? arrayBuffer.slice(0, totalBytesRead) : arrayBuffer;
  }
  async write(arrayBuffer) {
    let offset = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
    let length = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : arrayBuffer.byteLength;
    return new Promise((resolve, reject) => {
      const nOffset = Number(offset);
      const uint8Array = new Uint8Array(arrayBuffer, Number(offset), length);
      fs.write(this.handle, uint8Array, 0, length, nOffset, (err, bytesWritten) => err ? reject(err) : resolve(bytesWritten));
    });
  }
}
async function readBytes(fd, uint8Array, offset, length, position) {
  return await new Promise((resolve, reject) => fs.read(fd, uint8Array, offset, length, position, (err, bytesRead) => err ? reject(err) : resolve(bytesRead)));
}
//# sourceMappingURL=node-file.js.map