import { isBrowser } from "./utils/is-browser.js";
import { TextDecoder, TextEncoder } from "./text-encoder/text-encoder.js";
import { atob, btoa } from "./buffer/btoa.node.js";
import { encodeImageNode } from "./images/encode-image-node.js";
import { parseImageNode, NODE_FORMAT_SUPPORT } from "./images/parse-image-node.js";
import { NodeFile } from "./filesystems/node-file.js";
import { NodeFileSystem } from "./filesystems/node-filesystem.js";
import { fetchNode } from "./filesystems/fetch-node.js";
import { NodeHash } from "./crypto/node-hash.js";
import { versions } from 'node:process';
export const nodeVersion = parseInt(versions.node.split('.')[0]);
import { makeNodeStream } from "./streams/make-node-stream.js";
export { Blob_ as Blob } from "./file/install-blob-polyfills.js";
export { File_ as File } from "./file/install-file-polyfills.js";
if (isBrowser) {
  console.error('loaders.gl: The @loaders.gl/polyfills should only be used in Node.js environments');
}
globalThis.loaders = globalThis.loaders || {};
export { makeNodeStream } from "./streams/make-node-stream.js";
globalThis.loaders.makeNodeStream = makeNodeStream;
globalThis.loaders.NodeFile = NodeFile;
globalThis.loaders.NodeFileSystem = NodeFileSystem;
globalThis.loaders.fetchNode = fetchNode;
globalThis.loaders.NodeHash = NodeHash;
if (!globalThis.TextEncoder) {
  globalThis.TextEncoder = TextEncoder;
}
if (!globalThis.TextDecoder) {
  globalThis.TextDecoder = TextDecoder;
}
if (!('atob' in globalThis) && atob) {
  globalThis['atob'] = atob;
}
if (!('btoa' in globalThis) && btoa) {
  globalThis['btoa'] = btoa;
}
globalThis.loaders.encodeImageNode = encodeImageNode;
globalThis.loaders.parseImageNode = parseImageNode;
globalThis.loaders.imageFormatsNode = NODE_FORMAT_SUPPORT;
globalThis._parseImageNode = parseImageNode;
globalThis._imageFormatsNode = NODE_FORMAT_SUPPORT;
import { readFileAsArrayBuffer, readFileAsText, requireFromFile, requireFromString } from "./load-library/require-utils.node.js";
globalThis.loaders.readFileAsArrayBuffer = readFileAsArrayBuffer;
globalThis.loaders.readFileAsText = readFileAsText;
globalThis.loaders.requireFromFile = requireFromFile;
globalThis.loaders.requireFromString = requireFromString;
export { installFilePolyfills } from "./file/install-file-polyfills.js";
import { Headers as HeadersNode } from "./fetch/headers-polyfill.js";
import { Response as ResponseNode } from "./fetch/response-polyfill.js";
import { fetchNode as fetchNodePolyfill } from "./fetch/fetch-polyfill.js";
if (nodeVersion < 18) {
  if (!('Headers' in globalThis) && HeadersNode) {
    globalThis.Headers = HeadersNode;
  }
  if (!('Response' in globalThis) && ResponseNode) {
    globalThis.Response = ResponseNode;
  }
  if (!('fetch' in globalThis) && fetchNodePolyfill) {
    globalThis.fetch = fetchNodePolyfill;
  }
}
//# sourceMappingURL=index.js.map